/*jshint esnext:true */

//creates dynamic html data table
function create_html_table(array){
  var html = '<table id="db_data"><thead>';
  for(var i = 0; i < array.length; i++){
    var row = array[i];
    html += '<tr>';
    for(var j = 0; j < row.length; j++){
      html += i === 0 ? '<th onclick="sort_data(\'' + row[j] + '\')">' : '<td>';
      html += row[j];
      html += i === 0 ? '</th>' : '</td>';
    }
    html += i === 0 ? '</thead></tr><tbody>': '</tr>';
  }
  html += '</tbody></table>';
  return html;
}

//checks if element has a scrollbar
(function($) {
    $.fn.hasScrollBar = function() {
        return this.get(0).scrollHeight > this.height();
    };
})(jQuery);


//locks table header so it stays visible while scrolling
function add_table_scrolling(){

  var headercols = $('#db_data > thead > tr > th');
  headercols.each(i=>$(headercols[i]).css('width', $(headercols[i]).width()));

  var firstrow = $('#db_data > tbody > tr > td');
  firstrow.each(i=>$(firstrow[i]).css('width', $(firstrow[i]).width()));

  $('#db_data > tbody').css('width',$('#db_data > tbody').width()).css('display', 'block');
  $('#db_data > thead').css('width',$('#db_data > thead').width()).css('display', 'block');

  if($('#db_data > tbody').hasScrollBar()){
    $('#db_data > thead > tr > th').last().css('padding-right', 23);
  }
}


//combines the data from the selected database table with the dynamic html table
function build_selected_data_set(database_name, table_name, sort_by = null, reverse = null){
 google.script.run
  
  .withSuccessHandler(result=>{
   var array = JSON.parse(result);
   var html = create_html_table(array);
   if(sort_by !== null && reverse === null){
      html = html.replace("onclick=\"sort_data('" + sort_by + "')", "class=\"asc\" onclick=\"sort_data('" + sort_by + "', " + "true)");
   }else if(sort_by !== null){
      html = html.replace("onclick=\"sort_data('" + sort_by + "')", "class=\"desc\" onclick=\"sort_data()");
   }
   app.pages.databases.descendants.Data_HTML.html = html;
   add_table_scrolling();
   
 })
  
  .get_data(database_name, table_name, sort_by, reverse);
  
}


//rebuilds the table with the selected sort
function sort_data(sort_by = null, reverse = null){ 
  var database_name = app.pages.databases.descendants.Databases.datasource.item.database_name;
  var table_name = JSON.parse(app.pages.databases.descendants.Tables.datasource.item).table_name;
  build_selected_data_set(database_name, table_name, sort_by, reverse);
}


//changes date to local formatting
function dateFormat(dateString){
  var date = new Date(dateString);
  return date.toLocaleDateString();

}


//loads first table when selecting a database
function load_data_table(database){
  var page = app.pages.databases;

  page.descendants.Data_HTML.html = '';
  page.properties.tables = page.properties.unsortedTables = database.tables;
  
  var current_field_name = page.properties.tableSortField;
  var current_direction = page.properties.tableSortDirection;
  if(current_field_name) document.querySelector('.app-databases-' + current_field_name).classList.remove(current_direction);
  
  page.properties.tableSortDirection = page.properties.tableSortField = null;

  var database_name = database.database_name;
  var table_name = JSON.parse(database.tables[0]).table_name;

  page.descendants.Tables.datasource.load(
    ()=>{
      page.descendants.Tables.datasource.selectIndex(0);
      build_selected_data_set(database_name, table_name);
    }
  );
}

function set_order_by(field_name){
  var datasource = app.pages.databases.descendants.Databases.datasource;
  var current_direction = datasource.query.parameters.direction;
  var current_field_name = datasource.query.parameters.orderBy;
  var direction;
  datasource.properties.page = 0;
  
  if(current_field_name !== field_name){
    datasource.query.parameters.direction = direction = "asc";
    datasource.query.parameters.orderBy = field_name;
    
  }else if(current_direction === "asc"){
    datasource.query.parameters.direction = direction = "desc";
    
  }else{
    datasource.query.parameters.orderBy = null;
    datasource.query.parameters.direction = direction = null;
  }
  
  if(current_field_name) document.querySelector('.app-databases-' + current_field_name).classList.remove(current_direction);
  if(direction) document.querySelector('.app-databases-' + field_name).classList.add(direction);

  datasource.load(result =>{
     datasource.selectIndex(0);
     load_data_table(datasource.item); 
    });
}

  
  

function sort_table_table(field_name){
  var page =  app.pages.databases;
  var tables = page.properties.tables;
  var sorted_tables = tables.flat();
  var current_field_name = page.properties.tableSortField;
  var current_direction = page.properties.tableSortDirection;
  var direction;
  
  if(current_field_name !== field_name){
    page.properties.tableSortDirection = direction = "asc";
    page.properties.tableSortField = field_name;
    
  }else if(current_direction === "asc"){
    page.properties.tableSortDirection = direction = "desc";
    
  }else{
    page.properties.tableSortField = null;
    page.properties.tableSortDirection = direction = null;
  }
  
  if(page.properties.tableSortField === null){
    sorted_tables = page.properties.unsortedTables;
  }else{
  
    var mult = direction === 'asc' ? -1 : 1 ;

    sorted_tables.sort( (a, b) => {

      if(a === null) return 1 * mult;

      else if(b === null) return -1 * mult;

      var a_parsed = JSON.parse(a);
      var b_parsed = JSON.parse(b);

      if(a_parsed[field_name] == b_parsed[field_name]) return 0;

      else if(a_parsed[field_name] < b_parsed[field_name]) return 1 * mult;

      else return -1 * mult;

    });
  }
  
  if(current_field_name) document.querySelector('.app-databases-' + current_field_name).classList.remove(current_direction);
  if(direction) document.querySelector('.app-databases-' + field_name).classList.add(direction);

  page.properties.tables = sorted_tables;
  
  page.descendants.Tables.datasource.load(result=>{
    page.descendants.Tables.datasource.selectIndex(0);
    database_name = page.descendants.Databases.datasource.item.database_name;
    table_name = JSON.parse(page.descendants.Tables.datasource.item).table_name;
    build_selected_data_set(database_name, table_name);
  });
   
}




















